/*
Copyright (C)2006 George Ang (gnap.an AT gmail.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#include<kademlia/utils/debug.hpp>
#ifndef SESSION_HPP
#define SESSION_HPP

namespace kademlia
{
	template<typename _uintX, class debugger> class kad;
	template<typename _uintX, class debugger = debug>
		class session
		{
			public:
				typedef _uintX uint_type;

				session();

				virtual ~session();

				uint_type get_target_id() const;

				void set_target_id(uint_type const& val);

				uint_type get_distance() const;

				uint_type get_session_id();

				virtual void start() = 0;
				virtual void on_timer() = 0;
				virtual bool timed_out() = 0;
				virtual void prepare_to_stop() = 0;
				virtual bool is_finished() = 0;

			protected:
				int m_session_state;
				
				time_t m_created;
				time_t m_last_response;
				uint_type m_target;
				uint_type m_distance;
				uint_type m_session_id;
		};

	template<typename _uintX, class debugger>
		session<_uintX, debugger>::session()
		{
			m_created = time(NULL);
			m_last_response = time(NULL);
			m_session_id = session_manager<uint_type, debugger>::gen_session_id();

		}

	template<typename _uintX, class debugger>
		session<_uintX, debugger>::~session()
		{
		}

	template<typename _uintX, class debugger>
		typename session<_uintX, debugger>::uint_type
		session<_uintX, debugger>::get_target_id() const
		{
			return m_target;
		}

	template<typename _uintX, class debugger>
		void session<_uintX, debugger>::set_target_id(uint_type const& val)
		{
			m_target = val;
			m_distance = val;
			m_distance._xor(kad<uint_type, debugger>::get_kad_id());
		}

	template<typename _uintX, class debugger>
		typename session<_uintX, debugger>::uint_type
		session<_uintX, debugger>::get_distance() const
		{
			return m_distance;
		}

	template<typename _uintX, class debugger>
		typename session<_uintX, debugger>::uint_type
		session<_uintX, debugger>::get_session_id()
		{
			return m_session_id;
		}

};

#endif //SESSION_HPP
